@@ -21,6 +21,10 @@
     more info, see README
 """
 
+from __future__ import absolute_import, division, print_function
+from dragonlib.utils import six
+xrange = six.moves.xrange
+
 try:
     # Python 3
     import queue
@@ -37,8 +41,8 @@ import sys
 import threading
 import time
 import warnings
+import logging
 
-from dragonlib.utils.logging_utils import log
 from dragonpy.MC6809data.MC6809_data_raw2 import (
     OP_DATA, REG_A, REG_B, REG_CC, REG_D, REG_DP, REG_PC,
     REG_S, REG_U, REG_X, REG_Y
@@ -54,6 +58,8 @@ from dragonpy.utils.byte_word_values import signed8, signed16, signed5
 from dragonpy.utils.simple_debugger import print_exc_plus
 
 
+log = logging.getLogger(__name__)
+
 
 # HTML_TRACE = True
 HTML_TRACE = False
@@ -105,8 +111,17 @@ class CPUStatusThread(threading.Thread):
 
 
 class CPU(object):
+
+    SWI3_VECTOR = 0xfff2
+    SWI2_VECTOR = 0xfff4
+    FIRQ_VECTOR = 0xfff6
+    IRQ_VECTOR = 0xfff8
+    SWI_VECTOR = 0xfffa
+    NMI_VECTOR = 0xfffc
     RESET_VECTOR = 0xfffe
 
+    STARTUP_BURST_COUNT = 100
+
     def __init__(self, memory, cfg, cpu_status_queue=None):
         self.memory = memory
         self.cfg = cfg
@@ -114,6 +129,9 @@ class CPU(object):
         self.running = True
         self.cycles = 0
         self.last_op_address = 0 # Store the current run opcode memory address
+        self.burst_op_count = self.STARTUP_BURST_COUNT
+        self.sync_op_count = 100
+        self.max_burst_count = 10000
 
         if cpu_status_queue is not None:
             status_thread = CPUStatusThread(self, cpu_status_queue)
@@ -194,7 +212,7 @@ class CPU(object):
             REG_CC: self.cc.get(),
 
             "cycles": self.cycles,
-            "RAM":self.memory.ram._mem[:] # copy of RAM,
+            "RAM": tuple(self.memory._mem) # copy of array.array() values,
         }
 
     def set_state(self, state):
@@ -216,7 +234,7 @@ class CPU(object):
         self.cc.set(state[REG_CC])
 
         self.cycles = state["cycles"]
-        self.memory.ram._mem = state["RAM"][:] # copy of RAM
+        self.memory.load(address=0x0000, data=state["RAM"])
 
     ####
 
@@ -250,25 +268,24 @@ class CPU(object):
 #        log.info("\tset PC to $%x" % (ea))
         self.program_counter.set(ea)
 
+    ####
+
     def get_and_call_next_op(self):
         op_address, opcode = self.read_pc_byte()
         try:
             self.call_instruction_func(op_address, opcode)
         except Exception as err:
             try:
-                raise Exception("%s - address: $%04x - opcode: $%02x" % (err, op_address, opcode))
-            except TypeError:
-                raise Exception("%s - address: %r - opcode: %r" % (err, op_address, opcode))
-
+                msg = "%s - op address: $%04x - opcode: $%02x" % (err, op_address, opcode)
+            except TypeError: # e.g: op_address or opcode is None
+                msg = "%s - op address: %r - opcode: %r" % (err, op_address, opcode)
+            exception = err.__class__ # Use origin Exception class, e.g.: KeyError
+            six.reraise(exception, exception(msg), sys.exc_info()[2])
 
     def quit(self):
         log.critical("CPU quit() called.")
         self.running = False
 
-    # same_op_count = 0
-    # last_op_code = None
-    # last_trace_line = None
-    # trace_line_no = 0
     def call_instruction_func(self, op_address, opcode):
         self.last_op_address = op_address
         try:
@@ -281,13 +298,118 @@ class CPU(object):
         instr_func(opcode)
         self.cycles += cycles
 
-    def burst_run(self, count):
+
+    ####
+
+    quickest_sync_callback_cycles = None
+    sync_callbacks_cyles = {}
+    sync_callbacks = []
+    def add_sync_callback(self, callback_cycles, callback):
+        """ Add a CPU cycle triggered callback """
+        self.sync_callbacks_cyles[callback] = 0
+        self.sync_callbacks.append([callback_cycles, callback])
+        if self.quickest_sync_callback_cycles is None or \
+                        self.quickest_sync_callback_cycles > callback_cycles:
+            self.quickest_sync_callback_cycles = callback_cycles
+
+    def call_sync_callbacks(self):
+        """ Call every sync callback with CPU cycles trigger """
+        current_cycles = self.cycles
+        for callback_cycles, callback in self.sync_callbacks:
+            # get the CPU cycles count of the last call
+            last_call_cycles = self.sync_callbacks_cyles[callback]
+
+            if current_cycles - last_call_cycles > callback_cycles:
+                # this callback should be called
+
+                # Save the current cycles, to trigger the next call
+                self.sync_callbacks_cyles[callback] = self.cycles
+
+                # Call the callback function
+                callback(current_cycles - last_call_cycles)
+
+    def burst_run(self):
+        """ Run CPU as fast as Python can... """
         # https://wiki.python.org/moin/PythonSpeed/PerformanceTips#Avoiding_dots...
         get_and_call_next_op = self.get_and_call_next_op
-        for __ in range(count):
-            get_and_call_next_op()
 
-    def test_run(self, start, end):
+        for __ in xrange(self.burst_op_count):
+            for __ in xrange(self.sync_op_count):
+                get_and_call_next_op()
+
+            self.call_sync_callbacks()
+
+    delay = 0
+    def delayed_burst_run(self, target_cycles_per_sec):
+        """ Run CPU not faster than given speedlimit """
+        old_cycles = self.cycles
+        start_time = time.time()
+
+        self.burst_run()
+
+        is_duration = time.time() - start_time
+        new_cycles = self.cycles - old_cycles
+        try:
+            is_cycles_per_sec = new_cycles / is_duration
+        except ZeroDivisionError:
+            pass
+        else:
+            should_burst_duration = is_cycles_per_sec / target_cycles_per_sec
+            target_duration = should_burst_duration * is_duration
+            delay = target_duration - is_duration
+            if delay > 0:
+                if delay > 1:
+                    self.delay = 1
+                else:
+                    self.delay = delay
+                time.sleep(self.delay)
+
+        self.call_sync_callbacks()
+
+    def calc_new_count(self, burst_count, current_value, target_value):
+        """
+        >>> calc_new_count(burst_count=100, current_value=30, target_value=30)
+        100
+        >>> calc_new_count(burst_count=100, current_value=40, target_value=20)
+        75
+        >>> calc_new_count(burst_count=100, current_value=20, target_value=40)
+        150
+        """
+        log.critical(
+            "%i op count current: %.4f target: %.4f",
+            self.burst_op_count, current_value, target_value
+        )
+        try:
+            new_burst_count = float(burst_count) / float(current_value) * target_value
+            new_burst_count += 1 # At least we need one loop ;)
+        except ZeroDivisionError:
+            return burst_count * 2
+
+        if new_burst_count > self.max_burst_count:
+            return self.max_burst_count
+
+        return int((burst_count + new_burst_count) / 2)
+
+    def run(self, max_run_time=0.1, target_cycles_per_sec=None):
+        now = time.time
+
+        start_time = now()
+
+        if target_cycles_per_sec is not None:
+            # Run CPU not faster than given speedlimit
+            self.delayed_burst_run(target_cycles_per_sec)
+        else:
+            # Run CPU as fast as Python can...
+            self.delay = 0
+            self.burst_run()
+
+        # Calculate the burst_count new, to hit max_run_time
+        self.burst_op_count = self.calc_new_count(self.burst_op_count,
+            current_value=now() - start_time - self.delay,
+            target_value=max_run_time,
+        )
+
+    def test_run(self, start, end, max_ops=1000000):
 #        log.warning("CPU test_run(): from $%x to $%x" % (start, end))
         self.program_counter.set(start)
 #        log.debug("-"*79)
@@ -296,18 +418,33 @@ class CPU(object):
         get_and_call_next_op = self.get_and_call_next_op
         program_counter = self.program_counter.get
 
-        while program_counter() != end:
+        for __ in xrange(max_ops):
+            if program_counter() == end:
+                return
             get_and_call_next_op()
+        raise RuntimeError("Max ops %i arrived!" % max_ops)
+        log.critical("Max ops %i arrived!", max_ops)
 
     def test_run2(self, start, count):
 #        log.warning("CPU test_run2(): from $%x count: %i" % (start, count))
         self.program_counter.set(start)
 #        log.debug("-"*79)
-        self.burst_run(count)
+
+        _old_burst_count = self.burst_op_count
+        self.burst_op_count = count
+
+        _old_sync_count = self.sync_op_count
+        self.sync_op_count = 1
+
+        self.burst_run()
+
+        self.burst_op_count = _old_burst_count
+        self.sync_op_count = _old_sync_count
 
 
     ####
 
+
     @property
     def get_info(self):
         return "cc=%02x a=%02x b=%02x dp=%02x x=%04x y=%04x u=%04x s=%04x" % (
@@ -2387,6 +2524,50 @@ class CPU(object):
         raise NotImplementedError("$%x RESET" % opcode)
         # Update CC bits: *****
 
+    # ---- Interrupt handling ----
+
+    irq_enabled = False
+    def irq(self):
+        if not self.irq_enabled or self.cc.I == 1:
+            log.critical("$%04x *** IRQ, ignore!\t%s" % (
+                self.program_counter.get(), self.cc.get_info
+            ))
+            return
+
+        if self.cc.E:
+            self.push_irq_registers()
+        else:
+            self.push_firq_registers()
+
+        ea = self.memory.read_word(self.IRQ_VECTOR)
+        log.critical("$%04x *** IRQ, set PC to $%04x\t%s" % (
+            self.program_counter.get(), ea, self.cc.get_info
+        ))
+        self.program_counter.set(ea)
+
+    def push_irq_registers(self):
+        """
+        push PC, U, Y, X, DP, B, A, CC on System stack pointer
+        """
+        self.cycles += 1
+        self.push_word(self.system_stack_pointer, self.program_counter.get()) # PC
+        self.push_word(self.system_stack_pointer, self.user_stack_pointer.get()) # U
+        self.push_word(self.system_stack_pointer, self.index_y.get()) # Y
+        self.push_word(self.system_stack_pointer, self.index_x.get()) # X
+        self.push_byte(self.system_stack_pointer, self.direct_page.get()) # DP
+        self.push_byte(self.system_stack_pointer, self.accu_b.get()) # B
+        self.push_byte(self.system_stack_pointer, self.accu_a.get()) # A
+        self.push_byte(self.system_stack_pointer, self.cc.get()) # CC
+
+    def push_firq_registers(self):
+        """
+        FIRQ - Fast Interrupt Request
+        push PC and CC on System stack pointer
+        """
+        self.cycles += 1
+        self.push_word(self.system_stack_pointer, self.program_counter.get()) # PC
+        self.push_byte(self.system_stack_pointer, self.cc.get()) # CC
+
     @opcode(# Return from interrupt
         0x3b, # RTI (inherent)
     )
@@ -2401,7 +2582,33 @@ class CPU(object):
 
         CC bits "HNZVC": -----
         """
-        raise NotImplementedError("$%x RTI" % opcode)
+        cc = self.pull_byte(self.system_stack_pointer) # CC
+        self.cc.set(cc)
+        if self.cc.E:
+            self.accu_a.set(
+                self.pull_byte(self.system_stack_pointer) # A
+            )
+            self.accu_b.set(
+                self.pull_byte(self.system_stack_pointer) # B
+            )
+            self.direct_page.set(
+                self.pull_byte(self.system_stack_pointer) # DP
+            )
+            self.index_x.set(
+                self.pull_word(self.system_stack_pointer) # X
+            )
+            self.index_y.set(
+                self.pull_word(self.system_stack_pointer) # Y
+            )
+            self.user_stack_pointer.set(
+                self.pull_word(self.system_stack_pointer) # U
+            )
+
+        self.program_counter.set(
+            self.pull_word(self.system_stack_pointer) # PC
+        )
+#         log.critical("RTI to $%04x", self.program_counter.get())
+
 
     @opcode(# Software interrupt (absolute indirect)
         0x3f, # SWI (inherent)
@@ -2507,77 +2714,26 @@ class TypeAssert(CPU):
 # CPU = TypeAssert # Should be only activated for debugging!
 
 
-
-def start_CPU(cfg, use_bus, bus_socket_host, bus_socket_port):
-    log.info(
-        "Connect to internal socket bus I/O %s:%s", bus_socket_host, bus_socket_port
-    )
-
-    if use_bus:
-        bus = socket.socket()
-        bus.connect(
-            (bus_socket_host, bus_socket_port)
-        )
-        assert bus is not None
-        cfg.bus = bus
-
-    cpu = CPU(cfg)
-    cpu.reset()
-    try:
-        cpu.run()
-    except SystemExit:
-        log.critical("CPU has raised system exit, ok.")
-    except:
-        print_exc_plus()
+#------------------------------------------------------------------------------
 
 
 def test_run():
-    print("test run...")
+    import sys
+    import os
     import subprocess
-    cmd_args = [sys.executable,
+    cmd_args = [
+        sys.executable,
         os.path.join("..", "DragonPy_CLI.py"),
-#        "--verbosity=5",
-#         "--verbosity=10", # DEBUG
-        "--verbosity=20", # INFO
-#         "--verbosity=30", # WARNING
-#         "--verbosity=40", # ERROR
-#        "--verbosity=50", # CRITICAL/FATAL
-#
-#         '--log_formatter=%(filename)s %(funcName)s %(lineno)d %(message)s',
-#
-#         "--machine=sbc09",
-#          "--machine=Simple6809",
-#        "--machine=Dragon32",
+#        "--verbosity", "5",
+        "--log", "dragonpy.components.cpu6809,50",
 
-         "--machine=Multicomp6809",
-
-#         "--max=15000",
-#        "--max=1",
-
-        "--display_cycle",
+        "--machine", "Dragon32", "run",
+#        "--machine", "Vectrex", "run",
+#        "--max_ops", "1",
+#        "--trace",
     ]
     print("Startup CLI with: %s" % " ".join(cmd_args[1:]))
-    subprocess.Popen(cmd_args).wait()
-    sys.exit(0)
-
+    subprocess.Popen(cmd_args, cwd="..").wait()
 
 if __name__ == "__main__":
-    #
-    # TODO: Only needed for windows-multiprocessing-work-a-round:
-    #       start with script with subprocess.Popen()
-    #
-    #    See: dragonpy.core.DragonPy
-    #
-    from dragonpy.DragonPy_CLI import get_cli
-    cli = get_cli()
-
-    if cli.cfg.bus is None:
-        print("DragonPy cpu core")
-        print("Run DragonPy_CLI.py instead")
-        test_run()
-        sys.exit(0)
-
-    bus_socket_host = cli.cfg.bus_socket_host
-    bus_socket_port = cli.cfg.bus_socket_port
-
-    start_CPU(cli.cfg, True, bus_socket_host, bus_socket_port)
+    test_run()
