@@ -11,17 +11,22 @@
 """
 
 from __future__ import absolute_import, division, print_function
+from dragonlib.utils import six
+xrange = six.moves.xrange
 
 import os
 import sys
 import time
+import logging
 
 from basic_editor.editor import EditorWindow
-from dragonlib.utils.logging_utils import log
+
 from dragonpy.Dragon32 import dragon_charmap
 from dragonpy.Dragon32.dragon_charmap import get_charmap_dict
-from dragonpy.Dragon32.dragon_font import CHARS_DICT, TkFont
-from dragonpy.utils.humanize import locale_format_number
+from dragonpy.Dragon32.dragon_font import CHARS_DICT, TkImageFont
+from dragonpy.utils.humanize import locale_format_number, get_python_info
+
+log = logging.getLogger(__name__)
 
 
 try:
@@ -31,6 +36,7 @@ try:
     from tkinter import filedialog
     from tkinter import messagebox
     from tkinter import scrolledtext
+    from tkinter import font as TkFont
 except ImportError:
     # Python 2
     import Queue as queue
@@ -38,6 +44,7 @@ except ImportError:
     import tkFileDialog as filedialog
     import tkMessageBox as messagebox
     import ScrolledText as scrolledtext
+    import tkFont as TkFont
 
 
 class MC6847_TextModeCanvas(object):
@@ -56,17 +63,19 @@ class MC6847_TextModeCanvas(object):
         self.columns = 16
 
         scale_factor = 2  # scale the complete Display/Characters
-        self.tk_font = TkFont(CHARS_DICT, scale_factor)  # to generate PhotoImage()
+        self.tk_font = TkImageFont(CHARS_DICT, scale_factor)  # to generate PhotoImage()
 
         self.total_width = self.tk_font.width_scaled * self.rows
         self.total_height = self.tk_font.height_scaled * self.columns
 
+        foreground, background = dragon_charmap.get_hex_color(dragon_charmap.NORMAL)
         self.canvas = tkinter.Canvas(root,
             width=self.total_width,
             height=self.total_height,
             bd=0, # no border
             highlightthickness=0, # no highlight border
-            bg="#ff0000",
+#             bg="#ff0000",
+            bg="#%s" % background,
         )
 
         # Contains the map from Display RAM value to char/color:
@@ -80,8 +89,8 @@ class MC6847_TextModeCanvas(object):
 
         # Create all charachter images on the display and fill self.images_map:
         self.init_img = self.tk_font.get_char(char="?", color=dragon_charmap.INVERTED)
-        for row in range(self.rows + 1):
-            for column in range(self.columns + 1):
+        for row in xrange(self.rows + 1):
+            for column in xrange(self.columns + 1):
                 x = self.tk_font.width_scaled * row
                 y = self.tk_font.height_scaled * column
                 image_id = self.canvas.create_image(x, y,
@@ -116,12 +125,206 @@ class MC6847_TextModeCanvas(object):
         self.canvas.itemconfigure(image_id, image=image)
 
 
+class RuntimeCfg(object):
+    """
+    TODO: Load/save to ~/.DragonPy.ini
+    """
+    speedlimit = False
+    cycles_per_sec = 888625 # cycles/sec
+    sync_op_count = 100
+    max_run_time = 0.1
+    max_burst_count = 1000
+
+    def __setattr__(self, attr, value):
+        log.critical("Set RuntimeCfg %r to: %r" % (attr, value))
+        return object.__setattr__(self, attr, value)
+
+    def load(self):
+        raise NotImplementedError("TODO!")
+    def save(self):
+        raise NotImplementedError("TODO!")
+
+
+
+class BaseTkinterGUIConfig(object):
+    """
+    14.318180 Mhz crystal / 16 = 0.894886 MHz CPU frequency * 1000000 = 894886 cycles/sec
+    14.218000 Mhz crystal / 16 = 0.888625 MHz CPU frequency * 1000000 = 888625 cycles/sec
+
+    894886 cycles/sec - 888625 cycles/sec = 6261 cycles/sec slower
+    14.218000 Mhz crystal = 0.00000113 Sec or 1.12533408356e-06 us cycle time
+    """
+    def __init__(self, gui, runtime_cfg):
+        self.gui = gui
+        self.runtime_cfg = runtime_cfg
+
+        self.root = tkinter.Toplevel(self.gui.root)
+        self.root.geometry("+%d+%d" % (
+            self.gui.root.winfo_rootx() + self.gui.root.winfo_width(),
+            self.gui.root.winfo_y() # FIXME: Different on linux.
+        ))
+
+        row = 0
+
+        #
+        # Speedlimit check button
+        #
+        # self.check_value_speedlimit = tkinter.BooleanVar( # FIXME: Doesn't work with PyPy ?!?!
+        self.check_value_speedlimit = tkinter.IntVar(
+            value=self.runtime_cfg.speedlimit
+        )
+        self.checkbutton_speedlimit = tkinter.Checkbutton(self.root,
+            text="speedlimit", variable=self.check_value_speedlimit,
+            command=self.command_checkbutton_speedlimit
+        )
+        self.checkbutton_speedlimit.grid(row=row, column=0)
+
+        #
+        # Cycles/sec entry
+        #
+        self.cycles_per_sec_var = tkinter.IntVar(
+            value=self.runtime_cfg.cycles_per_sec
+        )
+        self.cycles_per_sec_entry = tkinter.Entry(self.root,
+            textvariable=self.cycles_per_sec_var,
+            width=8, # validate = 'key', validatecommand = vcmd
+        )
+        self.cycles_per_sec_entry.bind('<KeyRelease>', self.command_cycles_per_sec)
+        self.cycles_per_sec_entry.grid(row=row, column=1)
+
+        self.cycles_per_sec_label_var = tkinter.StringVar()
+        self.cycles_per_sec_label = tkinter.Label(
+            self.root, textvariable=self.cycles_per_sec_label_var
+        )
+        self.root.after_idle(self.command_cycles_per_sec) # Add Text
+        self.cycles_per_sec_label.grid(row=row, column=2)
+
+        row += 1
+
+        #
+        # CPU burst max running time - self.runtime_cfg.max_run_time
+        #
+        self.max_run_time_var = tkinter.DoubleVar(
+            value=self.runtime_cfg.max_run_time
+        )
+        self.max_run_time_entry = tkinter.Entry(self.root,
+            textvariable=self.max_run_time_var, width=8,
+        )
+        self.max_run_time_entry.bind('<KeyRelease>', self.command_max_run_time)
+        self.max_run_time_entry.grid(row=row, column=1)
+        self.max_run_time_label = tkinter.Label(self.root,
+            text="How long should a CPU Op burst loop take (max_run_time)"
+        )
+        self.max_run_time_label.grid(row=row, column=2, sticky=tkinter.W)
+
+        row += 1
+
+        #
+        # CPU sync OP count - self.runtime_cfg.sync_op_count
+        #
+        self.sync_op_count_var = tkinter.IntVar(
+            value=self.runtime_cfg.sync_op_count
+        )
+        self.sync_op_count_entry = tkinter.Entry(self.root,
+            textvariable=self.sync_op_count_var, width=8,
+        )
+        self.sync_op_count_entry.bind('<KeyRelease>', self.command_sync_op_count)
+        self.sync_op_count_entry.grid(row=row, column=1)
+        self.sync_op_count_label = tkinter.Label(self.root,
+            text="How many Ops should the CPU process before check sync calls e.g. IRQ (sync_op_count)"
+        )
+        self.sync_op_count_label.grid(row=row, column=2, sticky=tkinter.W)
+
+        row += 1
+
+        #
+        # max CPU burst op count - self.runtime_cfg.max_burst_count
+        #
+        self.max_burst_count_var = tkinter.IntVar(
+            value=self.runtime_cfg.max_burst_count
+        )
+        self.max_burst_count_entry = tkinter.Entry(self.root,
+            textvariable=self.max_burst_count_var, width=8,
+        )
+        self.max_burst_count_entry.bind('<KeyRelease>', self.command_max_burst_count)
+        self.max_burst_count_entry.grid(row=row, column=1)
+        self.max_burst_count_label = tkinter.Label(self.root,
+            text="Max CPU op burst count (max_burst_count)"
+        )
+        self.max_burst_count_label.grid(row=row, column=2, sticky=tkinter.W)
+
+        self.root.update()
+
+    def command_checkbutton_speedlimit(self, event=None):
+        self.runtime_cfg.speedlimit = self.check_value_speedlimit.get()
+
+    def command_cycles_per_sec(self, event=None):
+        try:
+            cycles_per_sec = self.cycles_per_sec_var.get()
+        except ValueError:
+            self.cycles_per_sec_var.set(self.runtime_cfg.cycles_per_sec)
+            return
+
+        self.cycles_per_sec_label_var.set(
+            "cycles/sec / 1000000 = %f MHz CPU frequency * 16 = %f Mhz crystal" % (
+                cycles_per_sec / 1000000,
+                cycles_per_sec / 1000000 * 16,
+            )
+        )
+
+        self.runtime_cfg.cycles_per_sec = cycles_per_sec
+
+    def command_sync_op_count(self, event=None):
+        """ CPU burst max running time - self.runtime_cfg.sync_op_count """
+        try:
+            sync_op_count = self.sync_op_count_var.get()
+        except ValueError:
+            sync_op_count = self.runtime_cfg.sync_op_count
+
+        if sync_op_count < 1:
+            sync_op_count = self.runtime_cfg.sync_op_count
+
+        self.runtime_cfg.sync_op_count = sync_op_count
+        self.sync_op_count_var.set(self.runtime_cfg.sync_op_count)
+
+    def command_max_burst_count(self, event=None):
+        """ max CPU burst op count - self.runtime_cfg.max_burst_count """
+        try:
+            max_burst_count = self.max_burst_count_var.get()
+        except ValueError:
+            max_burst_count = self.runtime_cfg.max_burst_count
+
+        if max_burst_count < 1:
+            max_burst_count = self.runtime_cfg.max_burst_count
+
+        self.runtime_cfg.max_burst_count = max_burst_count
+        self.max_burst_count_var.set(self.runtime_cfg.max_burst_count)
+
+    def command_max_run_time(self, event=None):
+        """ CPU burst max running time - self.runtime_cfg.max_run_time """
+        try:
+            max_run_time = self.max_run_time_var.get()
+        except ValueError:
+            max_run_time = self.runtime_cfg.max_run_time
+
+        self.runtime_cfg.max_run_time = max_run_time
+        self.max_run_time_var.set(self.runtime_cfg.max_run_time)
+
+    def focus(self):
+        # see: http://www.python-forum.de/viewtopic.php?f=18&t=34643 (de)
+        self.root.attributes('-topmost', True)
+        self.root.attributes('-topmost', False)
+        self.root.focus_force()
+        self.root.lift(aboveThis=self.gui.root)
+
+
 class BaseTkinterGUI(object):
     """
     The complete Tkinter GUI window
     """
     def __init__(self, cfg, display_queue, user_input_queue):
         self.cfg = cfg
+        self.runtime_cfg = RuntimeCfg()
 
         # Queue which contains "write into Display RAM" information
         # for render them in MC6847_TextModeCanvas():
@@ -130,13 +333,16 @@ class BaseTkinterGUI(object):
         # Queue to send keyboard inputs to CPU Thread:
         self.user_input_queue = user_input_queue
 
-        self.burst_count = 100
+        self.op_delay = 0
+        self.burst_op_count = 100
         self.cpu_after_id = None # Used to call CPU OP burst loop
         self.target_burst_duration = 0.1 # Duration how long should a CPU Op burst loop take
 
         self.init_statistics() # Called also after reset
 
         self.root = tkinter.Tk(className="DragonPy")
+#         self.root.config(font="Helvetica 16 bold italic")
+
         self.root.geometry("+%d+%d" % (
             self.root.winfo_screenwidth() * 0.1, self.root.winfo_screenheight() * 0.1
         ))
@@ -144,10 +350,23 @@ class BaseTkinterGUI(object):
         self.root.bind("<Key>", self.event_key_pressed)
         self.root.bind("<<Paste>>", self.paste_clipboard)
 
+        menu_tk_font = TkFont.Font(
+            family='Helvetica',
+#             family='clean',
+            size=11, weight='normal'
+        )
+
         self.status = tkinter.StringVar(value="startup %s...\n" % self.cfg.MACHINE_NAME)
         self.status_widget = tkinter.Label(
-            self.root, textvariable=self.status, text="Info:", borderwidth=1)
-        self.status_widget.grid(row=1, column=0, columnspan=2)
+            self.root, textvariable=self.status, text="Info:", borderwidth=1,
+            font=menu_tk_font
+        )
+        self.status_widget.grid(row=1, column=0)
+
+        self.python_info_label = tkinter.Label(
+            self.root, borderwidth=1, text=get_python_info(),
+        )
+        self.python_info_label.grid(row=2, column=0)
 
         self.menubar = tkinter.Menu(self.root)
 
@@ -164,6 +383,10 @@ class BaseTkinterGUI(object):
         self.cpu_menu.add_command(label="hard reset", command=self.command_cpu_hard_reset)
         self.menubar.add_cascade(label="6809", menu=self.cpu_menu)
 
+        self.config_window = None
+        self.menubar.add_command(label="config", command=self.command_config)
+        self.root.after(200, self.command_config) # FIXME: Only for developing: Open config on startup!
+
         # help menu
         helpmenu = tkinter.Menu(self.menubar, tearoff=0)
         helpmenu.add_command(label="help", command=self.menu_event_help)
@@ -171,10 +394,12 @@ class BaseTkinterGUI(object):
         self.menubar.add_cascade(label="help", menu=helpmenu)
 
     def init_statistics(self):
+        self.op_count = 0
         self.last_op_count = 0
         self.last_cpu_cycles = 0
         self.cpu_cycles_update_interval = 1 # Fequency for update GUI status information
         self.next_cpu_cycle_update = time.time() + self.cpu_cycles_update_interval
+        self.last_cycles_per_second = sys.maxsize
 
     def menu_event_about(self):
         messagebox.showinfo("DragonPy",
@@ -197,6 +422,19 @@ class BaseTkinterGUI(object):
 
     #-----------------------------------------------------------------------------------------
 
+    def close_config(self):
+        self.config_window.root.destroy()
+        self.config_window = None
+
+    def command_config(self):
+        if self.config_window is None:
+            self.config_window = BaseTkinterGUIConfig(self, self.runtime_cfg)
+            self.config_window.root.protocol("WM_DELETE_WINDOW", self.close_config)
+        else:
+            self.config_window.focus()
+
+    #-----------------------------------------------------------------------------------------
+
     def status_paused(self):
         self.status.set("%s paused.\n" % self.cfg.MACHINE_NAME)
 
@@ -230,7 +468,7 @@ class BaseTkinterGUI(object):
             self.user_input_queue.put(char)
 
     def wait_until_input_queue_empty(self):
-        for count in range(1, 10):
+        for count in xrange(1, 10):
             self.cpu_interval()
             if self.user_input_queue.empty():
                 log.critical("user_input_queue is empty, after %i burst runs, ok.", count)
@@ -259,70 +497,76 @@ class BaseTkinterGUI(object):
         char_or_code = event.char or event.keycode
         self.user_input_queue.put(char_or_code)
 
-    def calc_new_count(self, burst_count, current_value, target_value):
-        """
-        >>> calc_new_count(burst_count=100, current_value=30, target_value=30)
-        100
-        >>> calc_new_count(burst_count=100, current_value=40, target_value=20)
-        75
-        >>> calc_new_count(burst_count=100, current_value=20, target_value=40)
-        150
-        """
-        try:
-            a = float(burst_count) / float(current_value) * target_value
-        except ZeroDivisionError:
-            return burst_count * 2
-        return int(round((burst_count + a) / 2))
-
+    total_burst_duration = 0
+    cpu_interval_calls = 0
+    last_display_queue_qsize = 0
     def cpu_interval(self, interval=None):
-#        log.critical("enter cpu interval")
+        self.cpu_interval_calls += 1
+
+        if self.runtime_cfg.speedlimit:
+            # Run CPU not faster than speedlimit
+            target_cycles_per_sec = self.runtime_cfg.cycles_per_sec
+        else:
+            # Run CPU as fast as Python can...
+            target_cycles_per_sec = None
+
         start_time = time.time()
-
-        self.machine.run_cpu(self.burst_count)
-
-        now = time.time()
-        burst_duration = now - start_time
-
-        # Calculate the burst_count new, to hit self.target_burst_duration
-        self.burst_count = self.calc_new_count(self.burst_count,
-            current_value=burst_duration,
-            target_value=self.target_burst_duration,
+        self.machine.cpu.run(
+            max_run_time=self.runtime_cfg.max_run_time,
+            target_cycles_per_sec=target_cycles_per_sec,
         )
-#        log.critical("burst duration: %.3f sec. - new burst count: %i Ops", burst_duration, burst_count)
-
-        if now > self.next_cpu_cycle_update:
-
-            duration = now - self.next_cpu_cycle_update + self.cpu_cycles_update_interval
-            self.next_cpu_cycle_update = now + self.cpu_cycles_update_interval
-
-            new_cycles = self.machine.cpu.cycles - self.last_cpu_cycles
-            self.last_cpu_cycles = self.machine.cpu.cycles
-            cycles_per_second = int(new_cycles / duration)
-
-            new_ops = self.machine.op_count - self.last_op_count
-            self.last_op_count = self.machine.op_count
-            ops_per_second = int(new_ops / duration)
-
-            msg = (
-                "%s cycles/sec (Dragon 32 == 895.000cycles/sec)"
-                "\n%s ops/sec - burst duration: %.3f sec. - burst count: %s Ops"
-            ) % (
-                locale_format_number(cycles_per_second),
-                locale_format_number(ops_per_second),
-                burst_duration, locale_format_number(self.burst_count)
-            )
-            self.status.set(msg)
+        now = time.time()
+        self.total_burst_duration += (now - start_time)
 
+        self.last_display_queue_qsize = (self.last_display_queue_qsize + self.display_queue.qsize()) / 2
         self.process_display_queue()
 
         if interval is not None:
             if self.machine.cpu.running:
-    #            log.critical("queue cpu interval")
-    #            self.root.after_idle(self.cpu_interval, machine, burst_count, interval)
                 self.cpu_after_id = self.root.after(interval, self.cpu_interval, interval)
             else:
                 log.critical("CPU stopped.")
 
+    last_update = 0
+    def update_status_interval(self, interval=500):
+
+        # Update CPU settings:
+        self.machine.cpu.sync_op_count = self.runtime_cfg.sync_op_count
+        self.machine.cpu.max_burst_count = self.runtime_cfg.max_burst_count
+
+        new_cycles = self.machine.cpu.cycles - self.last_cpu_cycles
+        duration = time.time() - self.last_update
+
+        cycles_per_sec = new_cycles / duration
+
+        msg = (
+            "%s cylces/sec (~%s burst op count)\n"
+            "%i CPU interval calls, display queue qsize: %.2f"
+        ) % (
+            locale_format_number(cycles_per_sec),
+            locale_format_number(self.machine.cpu.burst_op_count),
+            self.cpu_interval_calls,
+            self.last_display_queue_qsize
+        )
+
+        if self.runtime_cfg.speedlimit:
+            msg += (
+                "\ncylces/sec diff: %s"
+            ) % (
+                locale_format_number(
+                    abs(self.runtime_cfg.cycles_per_sec - cycles_per_sec)
+                ),
+            )
+
+        self.status.set(msg)
+
+        self.last_cpu_cycles = self.machine.cpu.cycles
+        self.cpu_interval_calls = 0
+        self.burst_loops = 0
+        self.last_update = time.time()
+
+        self.root.after(interval, self.update_status_interval, interval)
+
     def process_display_queue(self):
         """
         consume all exiting "display RAM write" queue items and render them.
@@ -343,6 +587,8 @@ class BaseTkinterGUI(object):
     def mainloop(self, machine):
         self.machine = machine
 
+        self.update_status_interval(interval=500)
+
         self.cpu_interval(interval=1)
 
         log.critical("Start root.mainloop()")
@@ -365,7 +611,7 @@ class DragonTkinterGUI(BaseTkinterGUI):
             "%s - Text Display 32 columns x 16 rows" % machine_name)
 
         self.display = MC6847_TextModeCanvas(self.root)
-        self.display.canvas.grid(row=0, column=0, columnspan=2)  # , rowspan=2)
+        self.display.canvas.grid(row=0, column=0)
 
         self._editor_window = None
 
@@ -446,7 +692,19 @@ def test_run():
     cmd_args = [
         sys.executable,
         os.path.join("..", "DragonPy_CLI.py"),
-#        "--verbosity", "5",
+#         "--log_list",
+
+#         "--verbosity", " 1", # hardcode DEBUG ;)
+#         "--verbosity", "10", # DEBUG
+#         "--verbosity", "20", # INFO
+#         "--verbosity", "30", # WARNING
+#         "--verbosity", "40", # ERROR
+#         "--verbosity", "50", # CRITICAL/FATAL
+        "--verbosity", "99", # nearly all off
+
+#         "--log", "DragonPy.cpu6809,50;dragonpy.Dragon32.MC6821_PIA,40",
+#         "--log", "dragonpy.components.cpu6809,50",
+
         "--machine", "Dragon32", "run",
 #        "--machine", "Vectrex", "run",
 #        "--max_ops", "1",
@@ -456,4 +714,4 @@ def test_run():
     subprocess.Popen(cmd_args, cwd="..").wait()
 
 if __name__ == "__main__":
-    test_run()
\ No newline at end of file
+    test_run()
