@@ -8,7 +8,8 @@
 """
 
 from __future__ import absolute_import, division, print_function
-
+from dragonlib.utils import six
+xrange = six.moves.xrange
 
 import logging
 import sys
@@ -39,7 +40,7 @@ class Test6809_Arithmetic(BaseCPUTestCase):
         excpected_values += list(range(0, 5))
 
         self.cpu.accu_a.set(0x00) # start value
-        for i in range(260):
+        for i in xrange(260):
             self.cpu.cc.set(0x00) # Clear all CC flags
             self.cpu_test_run(start=0x1000, end=None, mem=[
                 0x8B, 0x01, # ADDA #$1 Immediate
@@ -82,7 +83,7 @@ class Test6809_Arithmetic(BaseCPUTestCase):
                 self.assertEqual(self.cpu.cc.C, 0)
 
     def test_ADDA1(self):
-        for i in range(260):
+        for i in xrange(260):
             self.cpu_test_run(start=0x1000, end=None, mem=[
                 0x8B, 0x01, # ADDA   #$01
             ])
@@ -174,7 +175,7 @@ loop:
         """
         excpected_values = [0] + list(range(255, 0, -1))
 
-        for a in range(256):
+        for a in xrange(256):
             self.cpu.cc.set(0x00)
 
             self.cpu_test_run(start=0x1000, end=None, mem=[
@@ -231,7 +232,7 @@ loop:
         excpected_values = [0] + list(range(255, 0, -1))
         address = 0x10
 
-        for a in range(256):
+        for a in xrange(256):
             self.cpu.cc.set(0x00)
 
             self.cpu.memory.write_byte(address, a)
@@ -281,7 +282,7 @@ loop:
         excpected_values += list(range(0, 5))
 
         self.cpu.memory.write_byte(0x4500, 0x0) # start value
-        for i in range(260):
+        for i in xrange(260):
             self.cpu.cc.set(0x00) # Clear all CC flags
             self.cpu_test_run(start=0x1000, end=None, mem=[
                 0x7c, 0x45, 0x00, # INC $4500
@@ -319,7 +320,7 @@ loop:
         excpected_values = list(range(1, 256))
         excpected_values += list(range(0, 5))
 
-        for i in range(260):
+        for i in xrange(260):
             self.cpu_test_run(start=0x1000, end=None, mem=[
                 0x5c, # INCB
             ])
@@ -389,7 +390,7 @@ loop:
         excpected_values += list(range(255, 250, -1))
 
         self.cpu.accu_a.set(0xff) # start value
-        for i in range(260):
+        for i in xrange(260):
             self.cpu.cc.set(0x00) # Clear all CC flags
             self.cpu_test_run(start=0x1000, end=None, mem=[
                 0x80, 0x01, # SUBA #$01
@@ -452,7 +453,7 @@ loop:
 
         self.cpu.memory.write_byte(0x4500, 0xff) # start value
         self.cpu.accu_a.set(0xff) # start value
-        for i in range(260):
+        for i in xrange(260):
             self.cpu.cc.set(0x00) # Clear all CC flags
             self.cpu_test_run(start=0x1000, end=None, mem=[
                 0x7A, 0x45, 0x00, # DEC $4500
@@ -489,7 +490,7 @@ loop:
             self.assertEqual(self.cpu.cc.C, 0)
 
     def test_DECA(self):
-        for a in range(256):
+        for a in xrange(256):
             self.cpu.cc.set(0x00)
             self.cpu.accu_a.set(a)
             self.cpu_test_run(start=0x1000, end=None, mem=[
@@ -650,7 +651,7 @@ loop:
         self.assertEqual(self.cpu.cc.C, 1)
 
     def test_DAA2(self):
-        for add in range(0xff):
+        for add in xrange(0xff):
             self.cpu.cc.set(0x00)
             self.cpu.accu_a.set(0x01)
             self.cpu_test_run(start=0x0100, end=None, mem=[
@@ -696,6 +697,8 @@ loop:
 
 
 if __name__ == '__main__':
+    from dragonlib.utils.logging_utils import setup_logging
+
     setup_logging(log,
 #        level=1 # hardcore debug ;)
 #        level=10 # DEBUG
